#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

import argparse
from .queries import retrieve_problem_severities, retrieve_problem_types, get_exec_blocks, \
    get_all_spls_from_exec_blocks, create_comments, delete_comments, modify_comments
from .connections import MDDBConnector


def parser() -> argparse.ArgumentParser:
    """
    Initialize argument parser for command-line arguments

    :return: Argument parser
    """
    parser = argparse.ArgumentParser(
        description="Workspaces data annotator", formatter_class=argparse.RawTextHelpFormatter
    )

    # Create a group to capture the action to be done; require that users select exactly one of this group
    action_group = parser.add_mutually_exclusive_group(required=True)
    action_group.add_argument(
        "-c",
        "--create",
        action='store_true',
        help="Create a comment for one or more datasets. Cannot be used with -d/--delete or -m/--modify.",
    )
    action_group.add_argument(
        "-m",
        "--modify",
        action='store_true',
        help="Modify a comment for one or more datasets. Cannot be used with -d/--delete or -c/--create.",
    )
    action_group.add_argument(
        "-d",
        "--delete",
        action='store_true',
        help="Delete a comment for one or more datasets. Cannot be used with -c/--create or -m/--modify.",
    )

    # Create a group to capture the data that will be searched by when querying the database; require at least one
    search_group = parser.add_argument_group()
    search_group.add_argument(
        "--project_code",
        action="store",
        help="A string representing the project code to search by.",
    )
    search_group.add_argument(
        "--date-range",
        nargs=2,
        action="store",
        help="Two strings representing beginning and end date to search by.",
    )
    search_group.add_argument(
        "--fsid",
        action="store",
        help="A string representing the FSID to search by",
    )
    search_group.add_argument(
        "--configuration",
        action="store",
        help="A string representing the configuration to search by.",
    )
    search_group.add_argument(
        "--band",
        action="store",
        help="A string representing the band to search by.",
    )

    # Create a group to capture other data that is not required at the start of the program
    optional_group = parser.add_argument_group()
    optional_group.add_argument(
        "--comment",
        action="store",
        help="A string representing the comment to add.",
    )
    optional_group.add_argument(
        "--old-comment",
        action="store",
        help="A string representing the old comment to search by. Only used with -m/--modify.",
    )
    optional_group.add_argument(
        "--problem-severity",
        action="store",
        help="Two strings representing beginning and end date to search by.",
    )
    optional_group.add_argument(
        "--problem-type",
        action="store",
        help="Two strings representing beginning and end date to search by.",
    )

    return parser


def prompt_for_input_from_db_list(db_list, prompt):
    """
    Accepts a list of id and description pairs and a sentence prompt to give to a user, and returns the user's choice
    from this list.

    :param db_list: a list of id and description tuples
    :param prompt: a string representing a question to ask the user
    :return: the user's choice from the passed iin options
    """
    output_string = prompt + "\n"
    for item in db_list:
        output_string += str(item[0]) + '.  ' + item[1] + '\n'

    return input(output_string)


def main():
    print("**********************************")
    print("* WELCOME TO THE DATA ANNOTATOR! *")
    print("**********************************")

    args = parser().parse_args()
    try:
        conn = MDDBConnector()
    except Exception as e:
        print(e)

    # Prompt for any missing data
    if not args.problem_severity:
        problem_severities = retrieve_problem_severities(conn)
        prompt = "A problem severity is required. Input the number for the value you wish to assign and hit enter:"
        args.problem_severity = prompt_for_input_from_db_list(problem_severities, prompt)
    if not args.problem_type:
        problem_types = retrieve_problem_types(conn)
        prompt = "A problem type is required. Input the number for the value you wish to assign and hit enter:"
        args.problem_type = prompt_for_input_from_db_list(problem_types, prompt)
    if not args.comment:
        prompt = "Please type your comment and hit enter:\n"
        args.comment = input(prompt)

    # Find the initial execution blocks, use them to find all relevant science product locators and proceed
    ebs = get_exec_blocks(args.project_code, args.date_range, args.fsid, args.band, args.configuration, conn)
    all_spls = get_all_spls_from_exec_blocks(ebs, conn)
    if args.create:
        create_comments(all_spls, args.comment, args.problem_severity, args.problem_type, conn)
    elif args.delete:
        delete_comments(all_spls, args.comment, args.problem_severity, args.problem_type, conn)
    elif args.modify:
        if not args.old_comment:
            prompt = "Please type the old comment you wish to modify:\n"
        args.old_comment = input(prompt)
        modify_comments(all_spls, args.comment, args.old_comment, args.problem_severity, args.problem_type, conn)

    conn.commit()
    conn.close()


if __name__ == '__main__':
    main()